@@ -1,17 +1,31 @@
 import hashlib
 import hmac
+import logging
 import re
-import urllib3
-import requests
 from datetime import datetime, timezone
 from email.utils import format_datetime
 
-from .neato import Neato  # For default Vendor argument
+import requests
+import urllib3
+from voluptuous import (
+    ALLOW_EXTRA,
+    All,
+    Any,
+    Extra,
+    MultipleInvalid,
+    Range,
+    Required,
+    Schema,
+)
+
 from .exceptions import NeatoRobotException, NeatoUnsupportedDevice
+from .neato import Neato  # For default Vendor argument
 
 # Disable warning due to SubjectAltNameWarning in certificate
 urllib3.disable_warnings(urllib3.exceptions.SubjectAltNameWarning)
 
+_LOGGER = logging.getLogger(__name__)
+
 SUPPORTED_SERVICES = ["basic-1", "minimal-2", "basic-2", "basic-3", "basic-4"]
 ALERTS_FLOORPLAN = [
     "nav_floorplan_load_fail",
@@ -19,6 +33,80 @@ ALERTS_FLOORPLAN = [
     "nav_floorplan_not_created",
 ]
 
+RESULT_SCHEMA = Schema(
+    Any(
+        "ok",
+        "invalid_json",
+        "bad_request",
+        "command_not_found",
+        "command_rejected",
+        "ko",
+        # Everything below this line is not documented, but still present
+        "not_on_charge_base",
+    )
+)
+STANDARD_SCHEMA = Schema(
+    {
+        "version": int,
+        "reqId": str,
+        Required("result"): RESULT_SCHEMA,
+        "data": {Extra: object},
+    },
+    extra=ALLOW_EXTRA,
+)
+STATE_SCHEMA = Schema(
+    {
+        "version": int,
+        "reqId": str,
+        Required("result"): RESULT_SCHEMA,
+        "data": {Extra: object},
+        Required("state"): int,
+        "action": int,
+        "error": Any(str, None),
+        "alert": Any(str, None),
+        "cleaning": {
+            "category": int,
+            "mode": int,
+            "modifier": int,
+            "navigationMode": int,
+            "spotWidth": int,
+            "spotHeight": int,
+        },
+        "details": {
+            "isCharging": bool,
+            "isDocked": bool,
+            "dockHasBeenSeen": bool,
+            "charge": All(int, Range(min=0, max=100)),
+            "isScheduleEnabled": bool,
+        },
+        "availableCommands": {
+            "start": bool,
+            "stop": bool,
+            "pause": bool,
+            "resume": bool,
+            "goToBase": bool,
+        },
+        Required("availableServices"): {
+            "findMe": str,
+            "generalInfo": str,
+            "houseCleaning": str,
+            "localStats": str,
+            "manualCleaning": str,
+            "maps": str,
+            "preferences": str,
+            "schedule": str,
+            "spotCleaning": str,
+            # Undocumented services
+            "IECTest": str,
+            "logCopy": str,
+            "softwareUpdate": str,
+            "wifi": str,
+        },
+        "meta": {"modelName": str, "firmware": str},
+    },
+    extra=ALLOW_EXTRA,
+)
+
 
 class Robot:
     """Data and methods for interacting with a Neato Botvac Connected vacuum robot"""
@@ -48,18 +136,24 @@ class Robot:
         self.traits = traits
         self.has_persistent_maps = has_persistent_maps
 
+        # pylint: disable=anomalous-backslash-in-string
         self._url = "{endpoint}/vendors/{vendor_name}/robots/{serial}/messages".format(
-            endpoint=re.sub(":\d+", "", endpoint),  # Remove port number
+            endpoint=re.sub(":\d+", "", endpoint),  # noqa: W605, Remove port number
             vendor_name=vendor.name,
             serial=self.serial,
         )
         self._headers = {"Accept": vendor.nucleo_version}
 
-        if self.service_version not in SUPPORTED_SERVICES:
+        # Check if service_version is supported
+        # We manually scan the state here to perform appropriate error handling
+        state = self.get_robot_state().json()
+        if (
+            "availableServices" not in state
+            or "houseCleaning" not in state["availableServices"]
+            or state["availableServices"]["houseCleaning"] not in SUPPORTED_SERVICES
+        ):
             raise NeatoUnsupportedDevice(
-                "Version "
-                + self.service_version
-                + " of service houseCleaning is not known"
+                "Service houseCleaning is not supported by your robot"
             )
 
     def __str__(self):
@@ -70,7 +164,7 @@ class Robot:
             self.traits,
         )
 
-    def _message(self, json):
+    def _message(self, json: dict, schema: Schema):
         """
         Sends message to robot with data from parameter 'json'
         :param json: dict containing data to send
@@ -86,11 +180,16 @@ class Robot:
                 headers=self._headers,
             )
             response.raise_for_status()
+            schema(response.json())
         except (
             requests.exceptions.ConnectionError,
             requests.exceptions.HTTPError,
         ) as ex:
             raise NeatoRobotException("Unable to communicate with robot") from ex
+        except MultipleInvalid as ex:
+            _LOGGER.warning(
+                "Invalid response from %s: %s. Got: %s", self._url, ex, response.json()
+            )
 
         return response
 
@@ -152,22 +251,31 @@ class Robot:
                 },
             }
 
-        response = self._message(json)
-        response_dict = response.json()
+        response = self._message(json, STATE_SCHEMA)
+        result = response.json().get("result", None)
+        alert = response.json().get("alert", None)
+        if result != "ok":
+            _LOGGER.warning(
+                "Result of robot.start_cleaning is not ok: %s, alert: %s", result, alert
+            )
 
         # Fall back to category 2 if we tried and failed with category 4
         if (
             category == 4
-            and (
-                "alert" in response_dict and response_dict["alert"] in ALERTS_FLOORPLAN
-            )
-            or (
-                "result" in response_dict
-                and response_dict["result"] == "not_on_charge_base"
-            )
+            and alert in ALERTS_FLOORPLAN
+            or result == "not_on_charge_base"
         ):
             json["params"]["category"] = 2
-            return self._message(json)
+            response_fallback = self._message(json, STATE_SCHEMA)
+            result = response_fallback.json().get("result", None)
+            alert = response_fallback.json().get("alert", None)
+            if result != "ok":
+                _LOGGER.warning(
+                    "Result of robot.start_cleaning is not ok after fallback: %s, alert: %s",
+                    result,
+                    alert,
+                )
+            return response_fallback
 
         return response
 
@@ -211,54 +319,57 @@ class Robot:
                 "params": {"category": 3, "navigationMode": 1},
             }
 
-        return self._message(json)
+        return self._message(json, STATE_SCHEMA)
 
     def pause_cleaning(self):
-        return self._message({"reqId": "1", "cmd": "pauseCleaning"})
+        return self._message({"reqId": "1", "cmd": "pauseCleaning"}, STATE_SCHEMA)
 
     def resume_cleaning(self):
-        return self._message({"reqId": "1", "cmd": "resumeCleaning"})
+        return self._message({"reqId": "1", "cmd": "resumeCleaning"}, STATE_SCHEMA)
 
     def stop_cleaning(self):
-        return self._message({"reqId": "1", "cmd": "stopCleaning"})
+        return self._message({"reqId": "1", "cmd": "stopCleaning"}, STATE_SCHEMA)
 
     def send_to_base(self):
-        return self._message({"reqId": "1", "cmd": "sendToBase"})
+        return self._message({"reqId": "1", "cmd": "sendToBase"}, STATE_SCHEMA)
 
     def get_robot_state(self):
-        return self._message({"reqId": "1", "cmd": "getRobotState"})
+        return self._message({"reqId": "1", "cmd": "getRobotState"}, STATE_SCHEMA)
 
     def enable_schedule(self):
-        return self._message({"reqId": "1", "cmd": "enableSchedule"})
+        return self._message({"reqId": "1", "cmd": "enableSchedule"}, STANDARD_SCHEMA)
 
     def disable_schedule(self):
-        return self._message({"reqId": "1", "cmd": "disableSchedule"})
+        return self._message({"reqId": "1", "cmd": "disableSchedule"}, STANDARD_SCHEMA)
 
     def get_schedule(self):
-        return self._message({"reqId": "1", "cmd": "getSchedule"})
+        return self._message({"reqId": "1", "cmd": "getSchedule"}, STANDARD_SCHEMA)
 
     def locate(self):
-        return self._message({"reqId": "1", "cmd": "findMe"})
+        return self._message({"reqId": "1", "cmd": "findMe"}, STANDARD_SCHEMA)
 
     def get_general_info(self):
-        return self._message({"reqId": "1", "cmd": "getGeneralInfo"})
+        return self._message({"reqId": "1", "cmd": "getGeneralInfo"}, STANDARD_SCHEMA)
 
     def get_local_stats(self):
-        return self._message({"reqId": "1", "cmd": "getLocalStats"})
+        return self._message({"reqId": "1", "cmd": "getLocalStats"}, STANDARD_SCHEMA)
 
     def get_preferences(self):
-        return self._message({"reqId": "1", "cmd": "getPreferences"})
+        return self._message({"reqId": "1", "cmd": "getPreferences"}, STANDARD_SCHEMA)
 
     def get_map_boundaries(self, map_id=None):
         return self._message(
-            {"reqId": "1", "cmd": "getMapBoundaries", "params": {"mapId": map_id}}
+            {"reqId": "1", "cmd": "getMapBoundaries", "params": {"mapId": map_id}},
+            STANDARD_SCHEMA,
         )
 
     def get_robot_info(self):
-        return self._message({"reqId": "1", "cmd": "getRobotInfo"})
+        return self._message({"reqId": "1", "cmd": "getRobotInfo"}, STANDARD_SCHEMA)
 
     def dismiss_current_alert(self):
-        return self._message({"reqId": "1", "cmd": "dismissCurrentAlert"})
+        return self._message(
+            {"reqId": "1", "cmd": "dismissCurrentAlert"}, STANDARD_SCHEMA
+        )
 
     @property
     def schedule_enabled(self):
